using System;
using System.Collections.Generic;
using System.IO.Compression;
using System.Linq;
using System.Xml.Linq;
using PV178.Homework3.Sample;

namespace PV178.Homework3
{
    /// <summary>
    /// Rozhran pro 3. domc kol PV178 / 2012.
    /// </summary>
    public interface IHomework3
    {
        /// <summary>
        /// Nate XML dokument ze souboru, kter byl zkomprimovn pomoc <see cref="GZipStream"/>. 
        /// Naten dokument se zpracuje a pevede na kolekci td Movie. Tuto kolekci si pak tda ulo do pamti (datov poloky), aby s n
        /// mohla dle pracovat.
        /// </summary>
        /// <param name="filename">Jmno souboru.</param>
        /// <remarks>
        /// Podvejte se na konstruktor tdy GZipStream a na vechna peten metody XDocument.Load.
        /// - FileStream
        /// - GZipStream
        /// - XDocument.Load
        /// </remarks>
        void LoadFromFile(string filename);


        /// <summary>
        /// Vrt 10 nejlepch film podle hodnocen divk (average rating).
        /// </summary>
        /// <remarks>LINQ: OrderBy, Take</remarks>
        /// <exception cref="InvalidOperationException">pokud seznam film nen naten.</exception>
        IEnumerable<Movie> GetBestMovies();


        /// <summary>
        /// Vrt filmy, jejich nzev obsahuje zadan (pod)etzec. Nebere ohled na velikost psmen.
        /// </summary>
        /// <param name="movieName">st jmna filmu. Na velikosti psmen nezle.</param>
        /// <remarks>
        /// Mete pout jedno z peten string.IndexOf (http://msdn.microsoft.com/en-us/library/ms224425.aspx)
        /// </remarks>
        /// <exception cref="InvalidOperationException">pokud seznam film nen naten.</exception>
        IEnumerable<Movie> FindMoviesByName(string movieName);


        /// <summary>
        /// Vrt celkovou stop filmotky (souet dlky u vech film).
        /// </summary>
        /// <remarks>LINQ: Sum; TimeSpan.FromSeconds</remarks>
        /// <returns>Souet dlek vech film.</returns>
        /// <exception cref="InvalidOperationException">Pokud seznam film nen naten.</exception>
        TimeSpan GetTotalRuntime();


        /// <summary>
        /// Vrac filmy seskupen podle roku vydn.
        /// </summary>
        /// <remarks>LINQ: GroupBy</remarks>
        /// <returns>Seznam dvojic (rok, seznam film natoench v tom roce).</returns>
        /// <exception cref="InvalidOperationException">Pokud seznam film nen naten.</exception>
        IEnumerable<YearMoviesPair> GetMoviesGroupedByYear();


        /// <summary>
        /// Vrat jmna vech nr vyskytujcch se ve filmotce.
        /// </summary>
        /// <remarks>
        /// LINQ: SelectMany (from .. from ...), Distinct
        /// </remarks>
        /// <returns></returns>
        IEnumerable<string> GetAllGenres();


        /// <summary>
        /// Spot poet kolik film spad do jednotlivch nr, a tyto informace vrt seazen podle potu film pro 20 nejpouvanjch nr.
        /// </summary>
        /// <returns>Dvojice (nzev nru, poet film)</returns>
        /// <remarks>
        /// LINQ: SelectMany (from .. from ...), GroupBy, OrderBy, Take.
        /// </remarks>
        /// <exception cref="InvalidOperationException">Pokud seznam film nen naten.</exception>
        IEnumerable<GenreStatisticsItem> CountMoviesInGenres();


        /// <summary>
        /// Vrt <paramref name="maxItems"/> nejlpe hodnocench film, u nich alespo jeden nr obsahuje zadan text. Na velikosti psmen nezle.
        /// </summary>
        /// <param name="maxItems">Maximln poet vrcench poloek.</param>
        /// <param name="textInGenre">Text, kter se mus vyskytnout v nzvu alespo jednoho nru. Na velikosti psmen nezle.</param>
        /// <remarks>
        /// Z film, pro kter plat, e alespo jeden z nr obsahuje zadan text, vybere prvnch {maxItems} poloek.
        /// - LINQ:  Where, Any, Take
        /// - etzce: string.IndexOf()
        /// </remarks>
        /// <exception cref="InvalidOperationException">Pokud seznam film nen naten.</exception>
        IEnumerable<Movie> GetBestMoviesWithGenreText(int maxItems, string textInGenre);


        /// <summary>
        /// Vrt seznam vech herc, kte se v souboru vyskytuj. Kad osoba se v seznamu mus vyskytnou prv jednou.
        /// </summary>
        /// <returns>Iterace vech herc.</returns>
        /// <remarks>
        /// Tady je dleit mt sprvn naprogramovan metody Equals a GetHashCode.
        /// LINQ: opertory SelectMany, Distinct.
        /// </remarks>
        /// <exception cref="InvalidOperationException">Pokud seznam film nen naten.</exception>
        IEnumerable<Person> GetAllActors();


        /// <summary>
        /// Vrac seznam vech herc, ale tak, e maj-li dva herci stejn jmno objev se na seznamu jen jeden z nich.
        /// Pro tento el vytvote tdu implementujc rozhran IEqualityComparer(of Person), kter bude mt dv osoby za
        /// stejn, prv kdy maj stejn jmno, a pouijte ji v opertoru Distinct.
        /// </summary>
        /// <returns></returns>
        /// <remarks>
        /// Rozhran IEqualityComparer(of Person), LINQ opertor Distinct, Select, ToArray.
        /// http://msdn.microsoft.com/en-us/library/ms132151.aspx
        /// </remarks>
        /// <exception cref="InvalidOperationException">Pokud seznam film nen naten.</exception>
        Person[] GetUniqueActorsNames();


        /// <summary>
        /// Vrac vechny filmy, ve kterch hraj vichni zadan herci.
        /// </summary>
        /// <remarks>
        /// Vrac vechny filmy, pro kter plat, e vichni herci ze zadanho seznamu hraj ve filmu (jsou v mezi herci filmu).
        /// LINQ: Where, All
        /// </remarks>
        /// <param name="actors">Seznam herc, kte musej hrt ve filmu.</param>
        /// <exception cref="InvalidOperationException">Pokud seznam film nen naten.</exception>
        IEnumerable<Movie> GetMoviesWithAllActors(params Person[] actors);


        /// <summary>
        /// Vrac prvek XML se seznamem vech film, kter jsou k danmu datu dostupn na DVD. Vracejte XML v nsledujc podob:
        /// <![CDATA[
        /// <movies-on-dvd>
        ///     <movie id="...." name="...." available-since="..." />
        ///     <movie id="...." name="...." available-since="..." />
        ///     <movie id="...." name="...." available-since="..." />
        ///     ...
        /// </movies-on-dvd>
        /// ]]>
        /// </summary>
        /// <param name="since">datum ke ktermu zjitujeme dostupnost film</param>
        /// <exception cref="InvalidOperationException">Pokud seznam film nen naten.</exception>
        XElement GetMoviesAvailableOnDvd(DateTime since);


        /// <summary>
        /// Zape do zadanho souboru vsledky 4 metod tohoto rozhran: 
        /// - GetTotalRuntime, 
        /// - CountMoviesInGenres, 
        /// - GetBestMovies,
        /// - GetMoviesAvailableOnDvd s parametrem 1.9.1999.
        /// </summary>
        /// <remarks>
        /// Vstupn soubor bude mt tuto podobu:
        /// <![CDATA[
        /// <?xml version="1.0" encoding="utf-8"?>
        /// <homework3>
        ///   <total-runtime>P862DT22H2M3S</total-runtime>
        ///   <top-10-movies>
        ///     <id>BVtuT</id>
        ///     <id>BValH</id>
        ///      ...
        ///   </top-10-movies>
        ///   <most-used-genres>
        ///     <genre name="Dramas">3165</genre>
        ///     <genre name="Foreign Movies">2168</genre>
        ///      ...
        ///   </most-used-genres>
        ///   <movies-on-dvd>
        ///     <movie id="...." name="...." available-since="..." />
        ///     <movie id="...." name="...." available-since="..." />
        ///     <movie id="...." name="...." available-since="..." />
        ///     ...
        ///   </movies-on-dvd>
        /// </homework3>
        /// ]]>
        /// </remarks>
        /// <param name="outputFile">jmno vstupnho souboru.</param>
        /// <remarks>
        /// Pokud jste si vimli divnho zpisu celkovho asu v ukzce, tak to je standardn zpis asovho seku
        /// podle normy ISO 8601 (http://en.wikipedia.org/wiki/ISO_8601#Durations)
        /// </remarks>
        /// <exception cref="InvalidOperationException">Pokud seznam film nen naten.</exception>
        void SaveStatisticsToXml(string outputFile);


        /// <summary>
        /// Metoda String.Join slou ke zkombinovn poloek vtu nebo pole do jednoho etzce:
        /// <code>
        /// int[] numbers = new[] { 1, 2, 3 };
        /// string commaSeparatedList = string.Join(",", numbers);
        /// commandSeparatedList === "1,2,3"
        /// </code>
        /// S pomoc LINQ opertoru Aggregate vytvote funkci, kter udl to sam (bez pouit String.Join).
        /// </summary>
        /// <param name="separator">Oddlova jednotlivch poloek ve vslednm etzci.</param>
        /// <param name="values">Poloky ke zetzen.</param>
        string AggregateJoin(string separator, IEnumerable<object> values);
    }
}